%% Copyright 2012 Soren Hauberg
%%
%% This program is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program.  If not, see <http://www.gnu.org/licenses/>.

function y = geodesic_system_global_gaussian (state, manifold)
  %% Basic info
  D = numel (state) / 2; % dimensionality of manifold
  mu = manifold.mu;
  M = manifold.M;
  R = numel (mu);
  
  %% Interpret state
  c = state (1:D); c = c (:); % point on curve
  cm = state ((D+1):end); cm = cm (:); % curve derivative

  %% Compute weighted metric tensors in the points on the curve
  [Ms, ~, Wt] = metric_tensor (c, manifold);

  %% Compute kron(c', c')^T dM/dc.
  dL = zeros (D, 1);
  switch (manifold.weight_scheme)
    % {  
    case 'global gaussian'
      Gamma = manifold.Gamma;
      for n = 2:N-1
        kron_n = kron (cm (:, n), cm (:, n));
        dMdc = 0;
        for r = 1:R
          pr = (mu {r}).' * M {r};
          wtr = Wt (r, n);
          inner = 0;
          for j = 1:R
            pj = (mu {j}).' * M {j};
            wtj = Wt (j, n);
            inner = inner + wtj * (pr - pj).' * Gamma;
          end % for
          dMdc = dMdc + wtr * (M {r} (:)) * inner;
        end % for
        dL (:, n) = -kron_n.' * dMdc;
      end % for
      dL = -dL ./ repmat (sum (Wt).^2, D, 1);
    % } 
    otherwise
      error ('compute_geodesic: unknown weight scheme ''%d''', manifold.weight_scheme);
  end % switch

  try
  ymm = -0.5 * dL / Ms;
  catch
    eig (Ms)
    dsada
%  ymm = -0.5 * dL * pinv (Ms);
  end
  ym = cm;
  y = [ym(:); ymm(:)];
end % function
